package com.ningdatech.pmapi.organization.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.organization.mapper.DingOrganizationMapper;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Lierbao
 * @since 2023-02-09
 */
@Service
public class DingOrganizationServiceImpl extends ServiceImpl<DingOrganizationMapper, DingOrganization> implements IDingOrganizationService {

    @Override
    public DingOrganization getByOrgCode(String orgCode) {
        if (StringUtils.isBlank(orgCode)) {
            return null;
        }
        return this.getOne(Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getOrganizationCode, orgCode));
    }

    @Override
    public DingOrganization getParentOrganization(String orgCode) {
        DingOrganization dingOrganization = getByOrgCode(orgCode);
        if (dingOrganization == null) {
            return null;
        }
        String parentCode = dingOrganization.getParentCode();
        return getByOrgCode(parentCode);
    }

    @Override
    public List<DingOrganization> listByCodes(Collection<String> codes) {
        if (CollectionUtil.isEmpty(codes)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<DingOrganization> query = Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getOrganizationCode, codes);
        return list(query);
    }

    @Override
    public List<String> listNameByCodes(Collection<String> codes) {
        LambdaQueryWrapper<DingOrganization> query = Wrappers.lambdaQuery(DingOrganization.class)
                .select(DingOrganization::getOrganizationName)
                .eq(DingOrganization::getOrganizationCode, codes);
        return CollUtils.fieldList(list(query), DingOrganization::getOrganizationName);
    }

    @Override
    public Map<String, String> getOrgCodeAndNameMap(List<String> orgCodeList) {
        orgCodeList = orgCodeList.stream().distinct().collect(Collectors.toList());
        List<DingOrganization> dingOrganizationList = listByCodes(orgCodeList);
        return dingOrganizationList.stream()
                .filter(r -> StringUtils.isNotBlank(r.getOrganizationCode()) && StringUtils.isNotBlank(r.getOrganizationName()))
                .collect(Collectors.toMap(DingOrganization::getOrganizationCode, DingOrganization::getOrganizationName));
    }
}
