package com.ningdatech.pmapi.projectdeclared.manage;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.pmapi.common.constant.CommonConst;
import com.ningdatech.pmapi.common.enumeration.ProjectProcessStageEnum;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.statemachine.util.StateMachineUtils;
import com.ningdatech.pmapi.common.util.ExcelDownUtil;
import com.ningdatech.pmapi.common.util.ExcelExportStyle;
import com.ningdatech.pmapi.projectdeclared.model.dto.DeclaredProjectExportDTO;
import com.ningdatech.pmapi.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.ningdatech.pmapi.projectdeclared.model.dto.PretrialDeclaredExportDTO;
import com.ningdatech.pmapi.projectdeclared.model.dto.ProjectConditionDTO;
import com.ningdatech.pmapi.projectdeclared.model.req.PrequalificationDeclaredListReq;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectTypeEnum;
import com.ningdatech.pmapi.projectlib.helper.ProjectHelper;
import com.ningdatech.pmapi.projectlib.manage.ProjectLibManage;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectInst;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.pmapi.projectlib.service.IProjectInstService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.staging.service.IProjectStagingService;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.wflow.bean.entity.WflowModels;
import com.wflow.exception.BusinessException;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import com.wflow.workflow.service.ProcessModelService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname ProqualificationDeclaredProjectManage
 * @Description
 * @Date 2023/2/13 14:48
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class PrequalificationDeclaredProjectManage {

    private final IProjectService projectService;

    private final StateMachineUtils stateMachineUtils;

    private final IProjectStagingService projectStagingService;

    private final ProjectLibManage projectLibManage;

    private final UserInfoHelper userInfoHelper;

    private final DefaultDeclaredProjectManage declaredProjectManage;

    private final DefaultDeclaredProjectManage defaultDeclaredProjectManage;

    /**
     * 提交预审
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String startTheProcess(DefaultDeclaredDTO dto) {
        UserInfoDetails userInfoDetails = LoginUserUtil.loginUserDetail();
        Long userId = userInfoDetails.getUserId();
        VUtils.isTrue(Objects.isNull(userId)).throwMessage("获取登录用户失败!");

        ProjectDTO projectDto = dto.getProjectInfo();
        VUtils.isTrue(Objects.isNull(projectDto.getId())).throwMessage("提交失败 缺少项目ID!");
        Project projectInfo = projectService.getById(projectDto.getId());
        VUtils.isTrue(Objects.isNull(projectInfo)).throwMessage("提交失败 此项目不存在!");

        //要判断 当前操作人 是不是项目主管单位的人
        VUtils.isTrue(!userInfoDetails.getOrganizationCode().equals(projectInfo.getSuperOrgCode()))
                .throwMessage(String.format("只有主管单位 【%s】的人 才能够提交",projectInfo.getSuperOrg()));

        //首先要判断 项目当前状态 是不是 待预审
        VUtils.isTrue(!ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode().equals(projectInfo.getStatus()) ||
                        !ProjectStatusEnum.NOT_APPROVED.getCode().equals(projectInfo.getStage()))
                .throwMessage("提交失败 该项目不是 待预审状态或者未立项阶段");
        //TODO 再判断 该项目是否 真实走完 单位内部审批

        //使用状态机 进入下一步 看看需不需要走省级审批
        stateMachineUtils.pass(projectInfo);
        String instanceId = null;
        //如果是省级部门 需要联审的(申报金额大于1000万 并且是市级项目)
        if(ProjectStatusEnum.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS
                .getCode().equals(projectInfo.getStatus())){
            //入库暂存表 后续处理 对接外部接口
            projectInfo.setUpdateOn(LocalDateTime.now());
            //保存一下  当前的主管单位发起人
            projectInfo.setPreStartUserId(userId);
            if(projectStagingService.addByProject(projectInfo,"省级部门联审")
                    && projectService.updateById(projectInfo)){
                return "提交省级部门联审成功";
            }
            return "提交省级部门联审失败";
        }else if(ProjectStatusEnum.PRE_APPLYING
                .getCode().equals(projectInfo.getStatus())){
            //如果是非省级联审的项目 直接提交 预审
            instanceId = defaultDeclaredProjectManage.directStartProcess(projectInfo,userId);
        }else{
            throw new BusinessException("项目状态 错误 project :" + JSON.toJSONString(projectInfo));
        }

        return "提交预审成功【" + instanceId + "】";
    }

    /**
     * 重新提交预审
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String restartTheProcess(DefaultDeclaredDTO dto) {
        UserInfoDetails userInfoDetails = LoginUserUtil.loginUserDetail();
        Long userId = userInfoDetails.getUserId();
        VUtils.isTrue(Objects.isNull(userId)).throwMessage("获取登录用户失败!");

        ProjectDTO projectDto = dto.getProjectInfo();
        VUtils.isTrue(Objects.isNull(projectDto.getId())).throwMessage("提交失败 缺少项目ID!");
        Project projectInfo = projectService.getById(projectDto.getId());
        VUtils.isTrue(Objects.isNull(projectInfo)).throwMessage("提交失败 此项目不存在!");
        //先回到 待预审状态
        projectInfo.setStatus(ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode());
        projectService.updateById(projectInfo);
        return startTheProcess(dto);
    }

    /**
     * 查询项目库
     * @param preReq
     * @return
     */
    public PageVo<ProjectLibListItemVO> projectLibList(PrequalificationDeclaredListReq preReq) {
        //限定参数 复制bean
        ProjectListReq req = new ProjectListReq();
        BeanUtils.copyProperties(preReq,req);
        //项目阶段 状态 已定 待预审
        req.setStage(ProjectStatusEnum.NOT_APPROVED.getCode());
        req.setStatus(ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode());
        Long userId = LoginUserUtil.getUserId();
        VUtils.isTrue(Objects.isNull(userId)).throwMessage("获取登录用户失败!");
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        //放入用户的主管单位
        req.setSuperOrgCode(userFullInfo.getOrganizationCode());
        return projectLibManage.projectLibList(req);
    }

    public void exportList(HttpServletResponse response, PrequalificationDeclaredListReq preReq) {
        ProjectListReq req = new ProjectListReq();
        BeanUtils.copyProperties(preReq,req);
        //项目阶段 状态 已定 待预审
        req.setStage(ProjectStatusEnum.NOT_APPROVED.getCode());
        req.setStatus(ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode());
        Long userId = LoginUserUtil.getUserId();
        VUtils.isTrue(Objects.isNull(userId)).throwMessage("获取登录用户失败!");
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        //放入用户的单位
        req.setBuildOrgCode(userFullInfo.getOrganizationCode());
        req.setPageNumber(CommonConst.EXPORT_PAGE_NUMBER);
        req.setPageSize(CommonConst.EXPORT_PAGE_SIZE);
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(req);
        Page<Project> page = projectService.page(req.page(), query);
        List<Project> records = page.getRecords();


        List<PretrialDeclaredExportDTO> collect = records.stream().map(r -> {
            PretrialDeclaredExportDTO exportDTO = new PretrialDeclaredExportDTO();
            BeanUtils.copyProperties(r, exportDTO);
            exportDTO.setProjectTypeName(ProjectTypeEnum.getDesc(r.getProjectType()));
            String createOnStr = NdDateUtils.format(r.getCreateOn(), "yyyy-MM-dd HH:mm");
            exportDTO.setCreateOn(createOnStr);
            return exportDTO;
        }).collect(Collectors.toList());
        for (int i = 0; i < collect.size(); i++) {
            collect.get(i).setSerialNumber(i + 1);
        }
        String fileName = "预审申报项目列表";
        ExcelDownUtil.setFileName(fileName,response);
        //数据导出处理函数
        try {
            EasyExcel.write(response.getOutputStream(), PretrialDeclaredExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(collect);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
